DOCUMENT_TYPES = {
    'final': 'final decision',
    'institution': 'institution decision',
    'petition': 'petition',
}


def get_ptab_document_types():
    return [v for k, v in DOCUMENT_TYPES.items()]



# flake8: noqa

# from src.silver_surfer_models.resources.PatentApplication import PatentApplicationModel
# from src.silver_surfer_models.resources.Trial.models import TrialModel
# from src.silver_surfer_models.resources.Patent.models import PatentModel
from src.silver_surfer_models.resources.File.models import FileModel



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...utils.utils import update_model
from .models import PriorArtTempModel
from .schemas import (
    PriorArtTempResourceSchema,
    PriorArtTempQueryParamsSchema,
    PriorArtTempPatchSchema,
)


schema_resource = PriorArtTempResourceSchema()
schema_params = PriorArtTempQueryParamsSchema()
schema_patch = PriorArtTempPatchSchema()


class DBException(SQLAlchemyError):
    pass


class PriorArtTempNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class PriorArtTemp:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                prior_art_id: int
                prior_art_detail: string
        :return:
            newly created object
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                prior_art_id
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        prior_art_temp_query = _build_query(params=data)
        response = schema_resource.dump(prior_art_temp_query, many=True).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: dict
                prior_art_detail
        :return:
            newly updated object
        :exception:
            PriorArtNotFoundException
            ValidationError
            DBException
        """
        prior_art_temp_query = db_session.query(
            PriorArtTempModel).filter_by(id=id).first()
        if not prior_art_temp_query:
            raise PriorArtTempNotFoundException('Prior art temp not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, prior_art_temp_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            PriorArtNotFoundException
            DBException
        """

        prior_art_temp_query = db_session.query(
            PriorArtTempModel).filter_by(id=id).first()
        if not prior_art_temp_query:
            raise PriorArtTempNotFoundException('Prior art does not exist!')
        try:
            db_session.delete(prior_art_temp_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')

    @staticmethod
    def bulk_delete(prior_art_id):
        try:
            db_session.query(PriorArtTempModel).filter(
                PriorArtTempModel.prior_art_id == prior_art_id
            ).delete(synchronize_session=False)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')


def _helper_create(data):
    new_prior_art_temp = PriorArtTempModel(
        prior_art_id=data['prior_art_id'],
        prior_art_detail=data['prior_art_detail'],
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_prior_art_temp)
        db_session.commit()
        prior_art_temp_query = db_session.query(
            PriorArtTempModel).get(
            new_prior_art_temp.id)
        response = schema_resource.dump(prior_art_temp_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise DBException('DB error')


def _helper_update(data, prior_art_temp_query):
    data['id'] = prior_art_temp_query.id
    data['prior_art_id'] = prior_art_temp_query.prior_art_id
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, prior_art_temp_query)
        db_session.commit()
        response = schema_resource.dump(prior_art_temp_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise DBException('DB error')


def _build_query(params):
    q = db_session.query(
        PriorArtTempModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('prior_art_id'):
        q = q.filter_by(prior_art_id=params.get('prior_art_id'))
    return q



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
)

from ...database import Base


class ProcessorResultModel(Base):
    __tablename__ = 'processor_results'

    id = Column(Integer, primary_key=True)
    document_id = Column(Integer, nullable=False)
    processor_name = Column(String(128), nullable=False)
    result = Column(Text, nullable=False)
    updated_at = Column(DateTime, nullable=False)



from marshmallow import (
    Schema,
    fields,
    pre_load,
    ValidationError,
)


class RelatedMatterResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    document_id = fields.Integer(required=True)
    pacer_case_id = fields.Integer()
    trial_id = fields.Integer()
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def pre_load(self, in_data):
        if 'pacer_case_id' in in_data and 'trial_id' in in_data:
            raise ValidationError(
                'Cannot provide both `pacer_case_id` and `trial_id` fields')

        if 'pacer_case_id' not in in_data and 'trial_id' not in in_data:
            raise ValidationError(
                'Should provide either `pacer_case_id` or `trial_id` field')


class RelatedMatterQueryParamsSchema(Schema):
    id = fields.Integer()
    document_id = fields.Integer()
    pacer_case_id = fields.Integer()
    trial_id = fields.Integer()



from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)

from src.silver_surfer_models.resources.LongTask import LongTask
from src.silver_surfer_models.resources.LongTask.schemas import (
    LONG_TASK_STATES,
    LONG_TASK_NAMES,
)
from test.backendtestcase import TestCase
from test.mixins.BasicDataMixin import BasicDataMixin
from test.utils import second_equals_first


class LongTaskResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(LongTaskResourceTestCase, self).setUp()
        self.inst = LongTask()

        self.long_task1 = self.inst.create({
            'trial_id': self.trial1['id'],
            'name': LONG_TASK_NAMES['PROCESS_PETITION_DOC'],
            'state': LONG_TASK_STATES['NOT_STARTED'],
            'progress': 0,
            'celery_task_id': '99999999-9999-9999-9999-999999999999',
        })
        self.long_task2 = self.inst.create({
            'trial_id': self.trial2['id'],
            'name': LONG_TASK_NAMES['PROCESS_PETITION_DOC'],
            'state': LONG_TASK_STATES['IN_PROGRESS'],
            'progress': 0,
            'celery_task_id': '88888888-8888-8888-8888-888888888888',
        })

    def test_create(self):
        data = {
            'trial_id': self.trial1['id'],
            'name': LONG_TASK_NAMES['PROCESS_PETITION_DOC'],
            'state': LONG_TASK_STATES['NOT_STARTED'],
            'progress': 0,
            'celery_task_id': '11111111-1111-1111-1111-111111111111'
        }
        resp = self.inst.create(data)
        second_equals_first(
            {
                'trial_id': self.trial1['id'],
                'name': data['name'],
                'state': data['state'],
                'progress': data['progress'],
                'celery_task_id': data['celery_task_id'],
            },
            resp,
        )

    def test_create_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    def test_read_w_params(self):
        resp = self.inst.read({
            'trial_id': self.trial1['id'],
        })
        self.assertEqual(len(resp), 1)
        second_equals_first(
            self.long_task1,
            resp[0],
        )

        resp = self.inst.read({
            'state': LONG_TASK_STATES['IN_PROGRESS'],
        })
        self.assertEqual(len(resp), 1)
        second_equals_first(
            self.long_task2,
            resp[0],
        )

        resp = self.inst.read({
            'celery_task_id': self.long_task2['celery_task_id'],
        })
        self.assertEqual(len(resp), 1)
        second_equals_first(
            self.long_task2,
            resp[0],
        )

    def test_one(self):
        resp = self.inst.one({
            'trial_id': self.trial1['id'],
        })
        second_equals_first(
            self.long_task1,
            resp,
        )

    def test_one_raises_NoResultFound(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': 999},
        )

    def test_one_raises_MultipleResultsFound(self):
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'name': LONG_TASK_NAMES['PROCESS_PETITION_DOC']},
        )

    def test_update(self):
        new_data = {
            'trial_id': self.trial1['id'],
            'name': LONG_TASK_NAMES['PROCESS_PETITION_DOC'],
            'state': LONG_TASK_STATES['IN_PROGRESS'],
            'progress': 0.50,
            'celery_task_id': 'some new',
            'result': 'something'
        }
        resp = self.inst.update(
            id=self.long_task1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.long_task1['id'],
                'trial_id': self.long_task1['trial_id'],
                'name': self.long_task1['name'],
                'state': new_data['state'],
                'progress': new_data['progress'],
                'celery_task_id': self.long_task1['celery_task_id'],
                'result': new_data['result'],
            },
            resp,
        )

    def test_delete(self):
        response = self.inst.read({'id': self.long_task1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.long_task1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.long_task1['id']})
        self.assertEqual(len(response), 0)



import copy

from marshmallow import ValidationError
from sqlalchemy.orm.exc import NoResultFound
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.silver_surfer_models.resources.Document import (
    Document,
)
from src.silver_surfer_models.resources.Trial import Trial
from src.silver_surfer_models.resources.PetitionDocumentReference import (
    PetitionDocumentReference,
)


class PetitionDocumentReferenceResourceTestCase(TestCase):
    def setUp(self):
        super(PetitionDocumentReferenceResourceTestCase, self).setUp()

        self.inst = PetitionDocumentReference()
        self.inst_trial = Trial()
        self.inst_document = Document()

        self.trial1 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-101',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.trial2 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': '1000',
            'is_cleaned': False,
            'is_processed': False,
        })

        self.document1 = self.inst_document.create({
            'trial_id': self.trial1['id'],
            'ptab_document_id': 1111,
            'title': 'Inter Partes Review Petition of US 7879828',
        })

        self.petition_document_reference1 = self.inst.create({
            'document_id': self.document1['id'],
            'type': 'CIVIL_CASE',
            'value': '1:16-cv-00469 (filed Feb. 26, 2016)',
        })

        self.petition_document_reference2 = self.inst.create({
            'document_id': self.document1['id'],
            'type': 'FEDERAL_CASE',
            'value': 'Something else',
        })

        self.valid_data = {
            'document_id': self.document1['id'],
            'type': 'FEDERAL_CASE',
            'value': '778 F.3d 1271, 1279',
        }

    @parameterized.expand([
        ('document_id',),
        ('type',),
        ('value',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        data = copy.copy(self.valid_data)
        data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(self.valid_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('document_id', 'document1', 'id', 2),
        ('type', 'petition_document_reference1', 'type', 1),
        ('value', 'petition_document_reference1', 'value', 1),
    ])
    def test_read_w_params(
            self,
            field_name,
            attr,
            attr_field,
            expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('type',),
        ('value',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.petition_document_reference1[field_name],
        })
        second_equals_first(
            self.petition_document_reference1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({
            'id': self.petition_document_reference1['id'],
        })
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.petition_document_reference1['id']},
        )

    def test_bulk_delete(self):
        response = self.inst.read({})
        self.assertEqual(2, len(response))

        self.inst.bulk_delete(document_id=self.document1['id'])

        response = self.inst.read({})
        self.assertEqual(0, len(response))



from sqlalchemy import (
    Boolean,
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class DocumentModel(Base):
    __tablename__ = 'documents'

    id = Column(Integer, primary_key=True)
    trial_id = Column(
        Integer,
        ForeignKey('trials.id'),
        nullable=False,
    )
    ptab_document_id = Column(Integer, nullable=False)
    title = Column(String(128))
    filing_date = Column(DateTime)
    type = Column(String(128))
    file_id = Column(Integer)
    has_smart_doc = Column(Boolean)
    exhibit_number = Column(String(128))
    document_number = Column(String(128))
    is_petition_doc = Column(Boolean, default=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (
        UniqueConstraint('ptab_document_id'),
        {'extend_existing': True},
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class PacerCaseResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    case_no = fields.String(required=True, validate=not_blank)
    court_id = fields.String(required=True, validate=not_blank)
    pacer_case_external_id = fields.String(required=True, validate=not_blank)
    cause = fields.String()
    county = fields.String()
    defendant = fields.String()
    disposition = fields.String()
    filed_date = fields.DateTime(allow_none=True)
    flags = fields.String()
    jurisdiction = fields.String()
    lead_case = fields.String()
    nature_of_suit = fields.String()
    office = fields.String()
    plaintiff = fields.String()
    related_case = fields.String()
    terminated_date = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)


class PacerCaseQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    ids = fields.List(fields.Integer())
    case_no = fields.String()
    court_id = fields.String()
    pacer_case_external_id = fields.String()


class PacerCasePatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    case_no = fields.String(validate=not_blank)
    court_id = fields.String(required=True, validate=not_blank)
    pacer_case_external_id = fields.String(required=True, validate=not_blank)
    cause = fields.String()
    county = fields.String()
    defendant = fields.String()
    filed_date = fields.DateTime()
    flags = fields.String()
    jurisdiction = fields.String()
    lead_case = fields.String()
    nature_of_suit = fields.String()
    office = fields.String()
    plaintiff = fields.String()
    related_case = fields.String()



from sqlalchemy import and_

from src.silver_surfer_models.resources.Trial.models import TrialModel
from .schemas import TrialResourceSchema
from src.silver_surfer_models.database import db_session

schema = TrialResourceSchema()


def get_trial_ids_final_to_download():
    trial_query = db_session.query(TrialModel).filter(
        TrialModel.final_doc_id == None)  # noqa
    response = schema.dump(trial_query, many=True).data
    return response


def get_trial_ids_institution_to_download():
    trial_query = db_session.query(TrialModel).filter(
        TrialModel.institution_doc_id == None)  # noqa
    response = schema.dump(trial_query, many=True).data
    return response


def get_trials_w_null_final_file_ids_and_nn_final_doc_ids():
    trial_query = db_session.query(TrialModel).filter(
        and_(
            TrialModel.final_doc_id != None,  # noqa
            TrialModel.final_file_id == None,  # noqa
        ))
    response = schema.dump(trial_query, many=True).data
    return response


def get_trials_w_null_insti_file_ids_and_nn_insti_doc_ids():
    trial_query = db_session.query(TrialModel).filter(
        and_(
            TrialModel.institution_doc_id != None,  # noqa
            TrialModel.institution_file_id == None,  # noqa
        ))
    response = schema.dump(trial_query, many=True).data
    return response



from marshmallow import (
    Schema,
    fields,
    validate,
    ValidationError,
)

LONG_TASK_STATES = {
    'NOT_STARTED': 'NOT_STARTED',
    'IN_PROGRESS': 'IN_PROGRESS',
    'SUCCESSFUL': 'SUCCESSFUL',
    'FAILED': 'FAILED',
}
LONG_TASK_NAMES = {
    'PROCESS_PETITION_DOC': 'PROCESS_PETITION_DOC',
}


def _validate_long_task_name(name):
    if name not in LONG_TASK_NAMES.keys():
        raise ValidationError(
            'Invalid long task name',
        )


def _validate_long_task_state(state):
    if state not in LONG_TASK_STATES.keys():
        raise ValidationError(
            'Invalid long task state',
        )


class LongTaskResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    trial_id = fields.Integer(required=True)
    name = fields.String(required=True, validate=_validate_long_task_name)
    state = fields.String(required=True, validate=_validate_long_task_state)
    progress = fields.Float(required=True)
    celery_task_id = fields.String(required=True)
    result = fields.String()
    updated_at = fields.DateTime()


class LongTaskQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    trial_id = fields.Integer()
    name = fields.String(validate=_validate_long_task_name)
    state = fields.String(validate=_validate_long_task_state)
    celery_task_id = fields.String(validate=not_blank)


class LongTaskPatchSchema(Schema):
    state = fields.String(validate=_validate_long_task_state)
    progress = fields.Float()
    result = fields.String()



from ...database import db_session
from ...resources.Document.models import DocumentModel
from ...resources.Document.schemas import DocumentResourceSchema

schema = DocumentResourceSchema()


def get_documents_to_download_by_type(document_type):
    """
    Args:
        document_type: string

    Returns: List

    """
    document_query = _build_query({
        'file_id': None,
        'type': document_type,
    })

    response = schema.dump(document_query, many=True).data
    return response


def get_documents_to_download():
    """
    Returns: List

    """
    document_query = _build_query({
        'file_id': None,
    })

    response = schema.dump(document_query, many=True).data
    return response


def get_documents_to_download_by_trial_id(trial_id):
    """
    Args:
        trial_id: int

    Returns: List

    """
    document_query = _build_query({
        'file_id': None,
        'trial_id': trial_id,
    })

    response = schema.dump(document_query, many=True).data
    return response


def _build_query(params):
    q = db_session.query(DocumentModel)
    if 'file_id' in params:
        q = q.filter(DocumentModel.file_id == params['file_id'])
    if 'type' in params:
        q = q.filter_by(type=params['type'])
    if 'trial_id' in params:
        q = q.filter_by(trial_id=params['trial_id'])
    return q


if __name__ == '__main__':
    res = get_documents_to_download()
    print(len(res))



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DocumentResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    trial_id = fields.Integer(required=True)
    ptab_document_id = fields.Integer(required=True)
    title = fields.String()
    filing_date = fields.DateTime()
    type = fields.String()
    file_id = fields.Integer()
    has_smart_doc = fields.Boolean()
    exhibit_number = fields.String()
    document_number = fields.String()
    is_petition_doc = fields.Boolean()
    updated_at = fields.DateTime(dump_only=True)


class DocumentQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    trial_id = fields.Integer()
    ptab_document_id = fields.Integer()
    document_number = fields.String()
    filing_date = fields.DateTime()
    type = fields.String()
    file_id = fields.Integer(allow_none=True)
    has_smart_doc = fields.Boolean()
    is_petition_doc = fields.Boolean()


class DocumentPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    trial_id = fields.Integer()
    ptab_document_id = fields.Integer()
    title = fields.String()
    filing_date = fields.DateTime()
    type = fields.String()
    file_id = fields.Integer()
    has_smart_doc = fields.Boolean()
    exhibit_number = fields.String()
    document_number = fields.String()
    is_petition_doc = fields.Boolean()



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...utils.utils import update_model
from ...resources.LongTask.models import LongTaskModel
from .schemas import (
    LongTaskResourceSchema,
    LongTaskQueryParamsSchema,
    LongTaskPatchSchema,
)


schema_resource = LongTaskResourceSchema()
schema_params = LongTaskQueryParamsSchema()
schema_patch = LongTaskPatchSchema()


class DBException(SQLAlchemyError):
    pass


class LongTaskNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class LongTask:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                trial_id: int
                name: string
                state: string
                progress: float
                celery_task_id: string
                result: string
        :return:
            newly created object
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                trial_id
                name
                state
                celery_task_id
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        long_task_query = _build_query(params=data)
        response = schema_resource.dump(long_task_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
            :param
                params: dict
                    id
                    trial_id
                    name
                    state
                    celery_task_id
            :return:
                queried object
            :exception:
                ValidationError
            """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        long_task_query = _build_query(params=data).one()
        response = schema_resource.dump(long_task_query).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: dict
                state: string
                progress: float
                result: string
        :return:
            newly updated object
        :exception:
            LongTaskNotFoundException
            ValidationError
            DBException
        """
        long_task_query = db_session.query(
            LongTaskModel).filter_by(id=id).first()
        if not long_task_query:
            raise LongTaskNotFoundException('LongTask not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, long_task_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            LongTaskNotFoundException
            DBException
        """

        long_task_query = db_session.query(
            LongTaskModel).filter_by(id=id).first()
        if not long_task_query:
            raise LongTaskNotFoundException('LongTask does not exist!')
        try:
            db_session.delete(long_task_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')


def _helper_create(data):
    new_long_task = LongTaskModel(
        trial_id=data['trial_id'],
        name=data['name'],
        state=data['state'],
        progress=data['progress'],
        celery_task_id=data['celery_task_id'],
        result=data.get('result', ''),
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_long_task)
        db_session.commit()
        long_task_query = db_session.query(
            LongTaskModel).get(new_long_task.id)
        response = schema_resource.dump(long_task_query).data
        db_session.close()
        return response
    except SQLAlchemyError as err:
        db_session.rollback()
        db_session.close()
        raise DBException(err)


def _helper_update(data, long_task_query):
    data['id'] = long_task_query.id
    data['trial_id'] = long_task_query.trial_id
    data['name'] = long_task_query.name
    data['celery_task_id'] = long_task_query.celery_task_id
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, long_task_query)
        db_session.commit()
        response = schema_resource.dump(long_task_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise DBException('DB error')


def _build_query(params):
    q = db_session.query(
        LongTaskModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('trial_id'):
        q = q.filter_by(trial_id=params.get('trial_id'))
    if params.get('name'):
        q = q.filter_by(name=params.get('name'))
    if params.get('state'):
        q = q.filter_by(state=params.get('state'))
    if params.get('celery_task_id'):
        q = q.filter_by(celery_task_id=params.get('celery_task_id'))
    return q



import re

from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.Document.models import DocumentModel
from .schemas import (
    DocumentResourceSchema,
    DocumentQueryParamsSchema,
    DocumentPatchSchema,
)
from ...utils.utils import update_model


schema_resource = DocumentResourceSchema()
schema_params = DocumentQueryParamsSchema()
schema_patch = DocumentPatchSchema()


class DBException(SQLAlchemyError):
    pass


class DocumentNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class Document:
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(DocumentResourceSchema)

        Returns: DocumentResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(DocumentQueryParamsSchema)

        Returns: List<DocumentResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        document_query = _build_query(params=data)
        response = schema_resource.dump(document_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(DocumentQueryParamsSchema)

        Returns: DocumentResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        document_query = _build_query(params=data).one()
        response = schema_resource.dump(document_query).data
        return response

    @staticmethod
    def update(id, params):
        """
        Args:
            id: int
            params: DocumentPatchSchema

        Returns: DocumentResourceSchema

        Raises:
            ValidationError
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        document_query = db_session.query(
            DocumentModel).filter_by(id=id).one()
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, document_query)
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        document_query = db_session.query(
            DocumentModel).filter_by(id=id).one()
        try:
            db_session.delete(document_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def upsert(params):
        """
        Args:
            params: DocumentResourceSchema

        Returns: DocumentResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'ptab_document_id': params['ptab_document_id'],
            }
            document_query = _build_query(query_params).one()
            response = _helper_update(data, document_query)
        except NoResultFound:
            response = _helper_create(data)
        return response

    @staticmethod
    def bulk_create(mappings):
        try:
            data, errors = schema_resource.load(mappings, many=True)
            db_session.bulk_insert_mappings(
                DocumentModel,
                data,
            )
            db_session.commit()
        except SQLAlchemyError:
            db_session.rollback()
            raise


def _helper_create(data):
    new_document = DocumentModel(**data)
    try:
        db_session.add(new_document)
        db_session.commit()
        document_query = db_session.query(
            DocumentModel).get(new_document.id)
        response = schema_resource.dump(document_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, document_query):
    data['id'] = document_query.id
    try:
        update_model(data, document_query)
        db_session.commit()
        response = schema_resource.dump(document_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        DocumentModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('trial_id'):
        q = q.filter_by(trial_id=params.get('trial_id'))
    if params.get('ptab_document_id'):
        q = q.filter_by(ptab_document_id=params.get('ptab_document_id'))
    if params.get('document_number'):
        q = q.filter_by(document_number=params.get('document_number'))
    if params.get('filing_date'):
        q = q.filter_by(filing_date=params.get('filing_date'))
    if params.get('type'):
        q = q.filter_by(type=params.get('type'))
        if params.get('type') == 'petition' and len(q.all()) > 1:
            petition_doc_ids = []
            for doc in q.all():
                if _title_contains_petition_word(doc):
                    petition_doc_ids.append(doc.id)

            if len(petition_doc_ids) > 1:
                q = q.filter_by(is_petition_doc=True)
            else:
                q = q.filter(DocumentModel.id.in_(petition_doc_ids))
    if 'file_id' in params:
        q = q.filter_by(file_id=params.get('file_id'))
    if 'has_smart_doc' in params:
        q = q.filter_by(has_smart_doc=params.get('has_smart_doc'))
    return q


def _title_contains_petition_word(document):
    match = re.search(r'petition(?:\s|$)', document.title, re.IGNORECASE)
    if document.type.lower() == 'petition' and match:
        return True
    return False



import os

import boto3
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from pymongo import MongoClient


def _get_ssm_parameters():
    result = {}
    session = boto3.Session(region_name='us-east-1')
    ssm = session.client('ssm')
    ssm_parameters = ssm.get_parameters(
        Names=[
            'mysqlHost',
            'mysqlUser',
            'mysqlPassword',
            'mysqlDB',
            'mongoHost',
            'mongoUser',
            'mongoPassword',
            'mongoDB',
        ],
        WithDecryption=True | False
    )['Parameters']
    for parameter in ssm_parameters:
        result[parameter['Name']] = parameter['Value']
    return result


def _get_credentials():
    result = {
        'mysqlHost': os.getenv('MYSQL_HOST', '127.0.0.1'),
        'mysqlUser': os.getenv('MYSQL_USER', 'root'),
        'mysqlPassword': os.getenv('MYSQL_PASSWORD', 'testpass'),
        'mysqlDB': os.getenv('MYSQL_DB', 'dev_db'),
        'mongoHost': os.getenv('MONGO_HOST', 'localhost'),
        'mongoUser': os.getenv('MONGO_USER', 'root'),
        'mongoPassword': os.getenv('MONGO_PASSWORD', 'testpass'),
        'mongoDB': os.getenv('MONGO_DB', 'silver-surfer'),
    }
    if os.getenv('FLASK_CONFIGURATION') == 'production':
        result = _get_ssm_parameters()
    return result


def _get_mysql_uri():
    environment = os.getenv('FLASK_CONFIGURATION')
    if environment == 'testing' or environment == 'ci':
        return 'sqlite://'

    credentials = _get_credentials()
    return 'mysql+pymysql://%s:%s@%s/%s?charset=utf8mb4' % (
        credentials['mysqlUser'],
        credentials['mysqlPassword'],
        credentials['mysqlHost'],
        credentials['mysqlDB'],
    )


def _get_mongo_db_uri_and_db_name():
    credentials = _get_credentials()
    db_name = credentials['mongoDB']
    db_uri = 'mongodb+srv://{}:{}@{}/{}?retryWrites=true'.format(
        credentials['mongoUser'],
        credentials['mongoPassword'],
        credentials['mongoHost'],
        db_name,
    )
    return db_uri, db_name


def import_models():
    # import all modules here that might define models so that
    # they will be registered properly on the metadata.  Otherwise
    # you will have to import them first before calling init_db()
    # this usually happens when a model declares a FK constraint but the
    # referenced table hasn't been created yet
    import src.silver_surfer_models.resources.all_models  # noqa


mysql_db_uri = _get_mysql_uri()
engine = create_engine(
    mysql_db_uri,
    convert_unicode=True,
)

db_session = scoped_session(
    sessionmaker(
        autocommit=False,
        autoflush=False,
        bind=engine,
    ),
)

Base = declarative_base()
Base.query = db_session.query_property()


def get_mongo_db_session():
    mongo_db_uri, mongo_db_name = _get_mongo_db_uri_and_db_name()
    mongo_client = MongoClient(mongo_db_uri)
    mongo_db_session = mongo_client[mongo_db_name]
    return mongo_db_session



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class OrangeBookPatentModel(Base):
    __tablename__ = 'orange_book_patents'

    id = Column(Integer, primary_key=True)
    appl_no = Column(String(128))
    appl_type = Column(String(128))
    delist_flag = Column(String(128))
    drug_product_flag = Column(String(128))
    drug_substance_flag = Column(String(128))
    patent_no = Column(String(128), nullable=False)
    patent_use_code = Column(String(128))
    product_no = Column(String(128), nullable=False)
    patent_expire_date = Column(DateTime)
    submission_date = Column(DateTime)
    updated_at = Column(DateTime)

    __table_args__ = (UniqueConstraint('patent_no', 'product_no'),)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class PetitionDocumentReferenceModel(Base):
    __tablename__ = 'petition_document_references'

    id = Column(Integer, primary_key=True)
    document_id = Column(
        Integer,
        ForeignKey('documents.id'),
        nullable=False,
    )
    type = Column(String(50), nullable=False)
    value = Column(String(255), nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import NoResultFound
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.silver_surfer_models.resources.Document import (
    Document,
)
from src.silver_surfer_models.resources.File import File
from src.silver_surfer_models.resources.Trial import Trial
from src.silver_surfer_models.resources.Document.document_types import (
    DOCUMENT_TYPES,
)


class DocumentResourceTestCase(TestCase):
    def setUp(self):
        super(DocumentResourceTestCase, self).setUp()
        self.inst = Document()
        self.inst_file = File()
        self.inst_trial = Trial()

        self.trial1 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-101',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.trial2 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': '1000',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.file1 = self.inst_file.create({
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key1123',
        })

        self.document1 = self.inst.create({
            'trial_id': self.trial1['id'],
            'ptab_document_id': 1111,
            'file_id': self.file1['id'],
            'type': DOCUMENT_TYPES['petition'],
            'title': 'Inter Partes Review Petition of US 7879828',
            'document_number': '2002',
        })
        self.document2 = self.inst.create({
            'trial_id': self.trial2['id'],
            'ptab_document_id': 2222,
            'type': DOCUMENT_TYPES['final'],
            'has_smart_doc': True,
        })
        self.document3 = self.inst.create({
            'trial_id': self.trial1['id'],
            'ptab_document_id': 3333,
            'type': DOCUMENT_TYPES['petition'],
            'title': 'Inter Partes Review Petition',
            'is_petition_doc': True,
        })

    def test_create_validation_error_no_trial_id(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'ptab_document_id': 1111,
                'title': 'Doc 1',
                'filing_date': '2017-07-05',
                'type': 'original_final',
            }
        )

    def test_create_validation_error_no_ptab_document_id(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'trial_id': self.trial1['id'],
                'title': 'Doc 1',
                'filing_date': '2017-07-05',
                'type': 'original_final',
            }
        )

    def test_create_fails_for_duplicate_ptab_document_id(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'trial_id': self.trial1['id'],
                'ptab_document_id': self.document1['ptab_document_id'],
                'title': 'Doc2',
                'filing_date': '2017-07-05',
                'type': DOCUMENT_TYPES['final'],
            }
        )

    def test_create(self):
        data = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 4444,
            'title': 'Doc2',
            'filing_date': '2017-07-05',
            'type': DOCUMENT_TYPES['final'],
            'exhibit_number': '10001',
        }
        resp = self.inst.create(data)
        second_equals_first(
            {
                'trial_id': self.trial1['id'],
                'ptab_document_id': data['ptab_document_id'],
                'title': data['title'],
                'filing_date': '2017-07-05T00:00:00+00:00',
                'type': data['type'],
                'file_id': None,
            },
            resp,
        )

    def test_read_all(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('trial_id', 'trial1', 'id', 2),
        ('ptab_document_id', 'document1', 'ptab_document_id', 1),
        ('file_id', 'file1', 'id', 1),
    ])
    def test_read_w_params1(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 3)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('type', DOCUMENT_TYPES['final'], 1),
        ('has_smart_doc', True, 1),
        ('file_id', None, 2),
    ])
    def test_read_w_params2(self, field_name, field_value, expected_length):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

        resp = self.inst.read({
            field_name: field_value,
        })
        self.assertEqual(expected_length, len(resp))
        second_equals_first(
            self.document2,
            resp[0],
        )

    def test_can_read_correct_petition_doc_when_multiple_petition_docs_present(
        self,
    ):
        petition_docs = self.inst.read({
            'trial_id': self.document1['trial_id'],
            'type': 'petition',
        })
        self.assertEqual(1, len(petition_docs))
        self.assertEqual(
            self.document3['id'],
            petition_docs[0]['id'],
        )

    def test_update(self):
        new_file = self.inst_file.create({
            'file_format': 'pdf',
            's3_bucket_name': 'test-bucket',
            's3_key_name': 'key1',
        })
        new_data = {
            'trial_id': self.trial2['id'],
            'title': 'New title',
            'type': DOCUMENT_TYPES['institution'],
            'file_id': new_file['id'],
            'has_smart_doc': True,
        }
        resp = self.inst.update(
            id=self.document1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.document1['id'],
                'trial_id': new_data['trial_id'],
                'ptab_document_id': self.document1['ptab_document_id'],
                'type': new_data['type'],
                'title': new_data['title'],
                'file_id': new_data['file_id'],
                'filing_date': None,
                'has_smart_doc': new_data['has_smart_doc'],
            },
            resp,
        )

    def test_update_has_smart_doc(self):
        new_data = {'has_smart_doc': True}
        resp = self.inst.update(
            id=self.document1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.document1['id'],
                'has_smart_doc': new_data['has_smart_doc'],
            },
            resp,
        )

    def test_update_exhibit_number(self):
        new_data = {'exhibit_number': '800001'}
        resp = self.inst.update(
            id=self.document1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.document1['id'],
                'exhibit_number': new_data['exhibit_number'],
            },
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.read({'id': self.document1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.document1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.document1['id']})
        self.assertEqual(len(response), 0)

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'trial_id': self.trial1['id'],
                'title': 'Doc 1',
                'filing_date': '2017-07-05',
                'type': 'original_final',
            }
        )

    def test_upsert_creates_new_entry(self):
        data = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 3333,
            'title': 'Doc2',
            'filing_date': '2017-07-05',
            'type': 'some-document-type',
            'document_number': '3000',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            {
                'trial_id': self.trial1['id'],
                'ptab_document_id': data['ptab_document_id'],
                'title': data['title'],
                'filing_date': '2017-07-05T00:00:00+00:00',
                'type': data['type'],
                'file_id': None,
                'document_number': data['document_number'],
            },
            resp,
        )

    def test_upsert_updates_existing_row(self):
        data = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 3333,
            'title': 'Doc2',
            'filing_date': '2017-07-05',
            'type': 'some-document-type',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            {
                'trial_id': self.trial1['id'],
                'ptab_document_id': data['ptab_document_id'],
                'title': data['title'],
                'filing_date': '2017-07-05T00:00:00+00:00',
                'type': data['type'],
                'file_id': None,
                'document_number': None,
            },
            resp,
        )

        new_data = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 3333,
            'title': 'Doc2',
            'filing_date': '2017-07-05',
            'type': 'other-document-type',
            'document_number': '3000',
        }
        response = self.inst.upsert(params=new_data)
        second_equals_first(
            {
                'trial_id': self.trial1['id'],
                'ptab_document_id': data['ptab_document_id'],
                'title': data['title'],
                'filing_date': '2017-07-05T00:00:00+00:00',
                'type': new_data['type'],
                'file_id': None,
                'document_number': new_data['document_number'],
            },
            response,
        )

    def test_bulk_create(self):
        mock_document1 = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 34545,
            'title': 'Some doc1',
            'filing_date': '2017-07-05',
            'type': 'other-document-type',
        }
        mock_document2 = {
            'trial_id': self.trial1['id'],
            'ptab_document_id': 345999,
            'title': 'Some doc2',
            'filing_date': '2017-07-05',
            'type': 'other-document-type',
        }
        mappings = [
            mock_document1,
            mock_document2,
        ]
        response = self.inst.read({})
        self.assertEqual(3, len(response))

        self.inst.bulk_create(mappings)
        response = self.inst.read({})
        self.assertEqual(5, len(response))



from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.silver_surfer_models.resources.Trial import Trial
from src.silver_surfer_models.resources.Document import Document
from src.silver_surfer_models.resources.PacerCase import PacerCase
from src.silver_surfer_models.resources.RelatedMatter import RelatedMatter
from src.silver_surfer_models.resources.Document.document_types import DOCUMENT_TYPES


class RelatedMatterResourceTestCase(TestCase):
    def setUp(self):
        super(RelatedMatterResourceTestCase, self).setUp()
        self.inst_trial = Trial()
        self.inst_document = Document()
        self.inst_pacer_case = PacerCase()
        self.inst = RelatedMatter()

        self.trial1 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-101',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.trial2 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-201',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.trial3 = self.inst_trial.create({
            'ptab_trial_num': 'CTA-301',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.document1 = self.inst_document.create({
            'trial_id': self.trial1['id'],
            'ptab_document_id': 2222,
            'type': DOCUMENT_TYPES['petition'],
            'has_smart_doc': True,
        })
        self.pacer_case1 = self.inst_pacer_case.create({
            'case_no': '3:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '2000',
            'cause': '15:1126 Patent Infringement',
        })
        self.pacer_case2 = self.inst_pacer_case.create({
            'case_no': '4:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '3000',
            'cause': '15:1126 Patent Infringement',
        })
        self.pacer_case3 = self.inst_pacer_case.create({
            'case_no': '5:17-cv-05314-SDW-LDW',
            'court_id': 'njd',
            'pacer_case_external_id': '4000',
            'cause': '15:1126 Patent Infringement',
        })

        self.related_matter1 = self.inst.create({
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case1['id'],
        })

        self.related_matter2 = self.inst.create({
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case2['id'],
        })

        self.related_matter3 = self.inst.create({
            'document_id': self.document1['id'],
            'trial_id': self.trial2['id'],
        })

    def _get_dict_with_fields(self, base_data, fields_to_keep):
        data = {}
        for key in base_data:
            if key in fields_to_keep:
                data[key] = base_data[key]
        return data

    @parameterized.expand([
        (['document_id'],),
        (['pacer_case_id', 'trial_id'],),
    ])
    def test_create_validation_error_missing_field(self, fields_to_keep):
        base_data = {
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case3['id'],
            'trial_id': self.trial3['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create_validation_error_both_fields(self):
        data = {
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case3['id'],
            'trial_id': self.trial3['id'],
        }
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['document_id', 'pacer_case_id'],),
        (['document_id', 'trial_id'],),
    ])
    def test_create_violates_unique_constraint(self, fields_to_keep):
        base_data = {
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case2['id'],
            'trial_id': self.trial2['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        ('document_id', ['pacer_case_id']),
        ('pacer_case_id', ['document_id', 'pacer_case_id']),
        ('trial_id', ['document_id', 'trial_id']),
    ])
    def test_create_violates_fk_constraint(
        self,
        field_to_test,
        fields_to_keep,
    ):
        base_data = {
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case2['id'],
            'trial_id': self.trial2['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        data[field_to_test] = 999

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['document_id', 'pacer_case_id'],),
        (['document_id', 'trial_id'],),
    ])
    def test_create(self, fields_to_keep):
        base_data = {
            'document_id': self.document1['id'],
            'pacer_case_id': self.pacer_case3['id'],
            'trial_id': self.trial3['id'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('id', 'related_matter1', 'id', 1),
        ('document_id', 'document1', 'id', 3),
        ('pacer_case_id', 'pacer_case1', 'id', 1),
        ('trial_id', 'trial2', 'id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 3)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_one_raises_no_result_found_exception(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': 9999},
        )

    def test_one_raises_multiple_results_found_exception(self):
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'document_id': self.document1['id']},
        )

    @parameterized.expand([
        ('id', 'related_matter1'),
        ('pacer_case_id', 'related_matter1'),
        ('trial_id', 'related_matter3'),
    ])
    def test_one(self, field_name, attr):
        related_matter = getattr(self, attr)
        resp = self.inst.one({
            field_name: related_matter[field_name],
        })
        second_equals_first(
            related_matter,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.related_matter1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.related_matter1['id']},
        )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class PatentModel(Base):
    __tablename__ = 'patents'

    id = Column(Integer, primary_key=True)
    patent_number = Column(String(128), nullable=False)
    jurisdiction = Column(String(128), nullable=False)
    app_grp_art_number = Column(String(128))
    country_code = Column(String(128))
    document_number = Column(String(128))
    kind_code = Column(String(128))
    primary_identifier = Column(String(256))
    abstract_text = Column(Text)
    applicant = Column(String(256))
    inventors = Column(String(256))
    title = Column(String(256))
    url = Column(String(500))
    grant_date = Column(DateTime)
    submission_date = Column(DateTime)
    app_filing_date = Column(DateTime)
    app_status_date = Column(DateTime)
    app_early_pub_date = Column(DateTime)
    pto_adjustments = Column(String(10))
    appl_delay = Column(String(10))
    total_pto_days = Column(String(10))
    assignee = Column(Text)
    patent_pdf_url = Column(String(500))

    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('patent_number', 'jurisdiction'),)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Float,
    ForeignKey,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class LongTaskModel(Base):
    __tablename__ = 'long_tasks'

    id = Column(Integer, primary_key=True)
    trial_id = Column(
        Integer,
        ForeignKey('trials.id'),
        nullable=False,
    )
    name = Column(String(128), nullable=False)
    state = Column(String(128), nullable=False)
    progress = Column(Float, nullable=False)
    celery_task_id = Column(String(128), nullable=False)
    result = Column(String(1024))
    updated_at = Column(DateTime, nullable=False)

    __table_args__ = (UniqueConstraint('celery_task_id'),)



from sqlalchemy import (
    Column,
    Integer,
    Text,
    DateTime,
    ForeignKey,
)

from ...database import Base


class PriorArtTempModel(Base):
    __tablename__ = 'prior_arts_temp'

    id = Column(Integer, primary_key=True)
    prior_art_id = Column(
        Integer,
        ForeignKey('prior_arts.id'),
        nullable=False,
    )
    prior_art_detail = Column(Text)
    updated_at = Column(DateTime, nullable=False)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()


INSTALL_REQUIRES = [
    'marshmallow==2.10.5',
    'PyMySQL==0.9.3',
    'python-dateutil==2.8.0',
    'pytz==2019.1',
    'SQLAlchemy==1.2.15',
    'boto3==1.9.134',
    'botocore==1.12.134',
    's3transfer==0.2.0',
    'six==1.12.0',
    'urllib3==1.24.2',
    'dnspython==1.16.0',
    'pymongo==3.8.0',
]

if __name__ == '__main__':
    setuptools.setup(
        name='silver-surfer-models',
        version='0.0.32-dev',
        author='Harsh Verma',
        author_email='harsh376@gmail.com',
        description='MySQL db models',
        long_description=long_description,
        long_description_content_type='text/markdown',
        url='https://github.com/harsh376/silver-surfer-models',
        packages=setuptools.find_packages(where='src'),
        package_dir={'': 'src'},
        classifiers=[
            'Programming Language :: Python :: 3',
        ],
        install_requires=INSTALL_REQUIRES,
        python_requires='~=3.6',
    )



from sqlalchemy import (
    Boolean,
    Column,
    Integer,
    String,
    DateTime,
)

from ...database import Base


class TrialModel(Base):
    __tablename__ = 'trials'
    __table_args__ = {'extend_existing': True}

    id = Column(Integer, primary_key=True)
    ptab_trial_num = Column(String(128), nullable=False, unique=True)
    trial_filing_date = Column(DateTime, nullable=False)
    patent_num = Column(String(128), nullable=False)

    accorded_filing_date = Column(DateTime)
    institution_decision_date = Column(DateTime)
    application_number = Column(String(128))
    inventor_name = Column(String(500))
    patent_owner_name = Column(String(500))
    petitioner_party_name = Column(String(500))
    prosecution_status = Column(String(128))

    is_cleaned = Column(Boolean, nullable=False)
    is_processed = Column(Boolean, nullable=False)
    updated_at = Column(DateTime, nullable=False)



from marshmallow import (
    Schema,
    fields,
    validate,
    ValidationError,
    pre_load,
)
from ...utils.utils import convert_string_to_datetime


def _pre_load_datetime_fields(in_data):
    try:
        if in_data.get('trial_filing_date'):
            in_data['trial_filing_date'] = convert_string_to_datetime(
                in_data['trial_filing_date'],
            )
        if in_data.get('accorded_filing_date'):
            in_data['accorded_filing_date'] = convert_string_to_datetime(
                in_data['accorded_filing_date'],
            )
        else:
            in_data['accorded_filing_date'] = None
        if in_data.get('institution_decision_date'):
            in_data['institution_decision_date'] = convert_string_to_datetime(
                in_data['institution_decision_date'],
            )
        else:
            in_data['institution_decision_date'] = None
    except (TypeError, ValueError):
        raise ValidationError('Invalid format: filing date')
    return in_data


class TrialResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    ptab_trial_num = fields.String(validate=not_blank, required=True)
    trial_filing_date = fields.DateTime(required=True)
    patent_num = fields.String(validate=not_blank, required=True)
    accorded_filing_date = fields.DateTime(allow_none=True)
    institution_decision_date = fields.DateTime(allow_none=True)
    application_number = fields.String(allow_none=True)
    inventor_name = fields.String(allow_none=True)
    patent_owner_name = fields.String(allow_none=True)
    petitioner_party_name = fields.String(allow_none=True)
    prosecution_status = fields.String(allow_none=True)
    is_cleaned = fields.Boolean()
    is_processed = fields.Boolean()
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        return _pre_load_datetime_fields(in_data)


class TrialQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    ids = fields.List(fields.Integer())
    ptab_trial_num = fields.String(validate=not_blank)
    trial_filing_date = fields.DateTime()
    patent_num = fields.String()
    patent_nums = fields.List(fields.String())
    is_cleaned = fields.Boolean()
    is_processed = fields.Boolean()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        return _pre_load_datetime_fields(in_data)


class TrialPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    ptab_trial_num = fields.String(validate=not_blank)
    trial_filing_date = fields.DateTime()
    patent_num = fields.String()
    accorded_filing_date = fields.DateTime(allow_none=True)
    institution_decision_date = fields.DateTime(allow_none=True)
    application_number = fields.String(allow_none=True)
    inventor_name = fields.String(allow_none=True)
    patent_owner_name = fields.String(allow_none=True)
    petitioner_party_name = fields.String(allow_none=True)
    prosecution_status = fields.String(allow_none=True)
    is_cleaned = fields.Boolean()
    is_processed = fields.Boolean()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        return _pre_load_datetime_fields(in_data)



import re

from sqlalchemy.exc import SQLAlchemyError
from marshmallow import ValidationError

from ...database import db_session
from ...resources.PetitionDocumentReference.models import (
    PetitionDocumentReferenceModel,
)
from .schemas import (
    PetitionDocumentReferenceResourceSchema,
    PetitionDocumentReferenceQueryParamsSchema,
)


schema_resource = PetitionDocumentReferenceResourceSchema()
schema_params = PetitionDocumentReferenceQueryParamsSchema()


class PetitionDocumentReference(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(PetitionDocumentReferenceResourceSchema)

        Returns: PetitionDocumentReferenceResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(PetitionDocumentReferenceQueryParamsSchema)

        Returns: List<PetitionDocumentReferenceResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        petition_document_ref_query = _build_query(params=data)
        response = schema_resource.dump(
            petition_document_ref_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(PetitionDocumentReferenceQueryParamsSchema)

        Returns: PetitionDocumentReferenceResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        petition_document_ref_query = _build_query(params=data).one()
        response = schema_resource.dump(petition_document_ref_query).data
        return response

    @staticmethod
    def delete(id):
        """
        Args:
            id: int

        Returns: string

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """
        petition_document_ref_query = db_session.query(
            PetitionDocumentReferenceModel).filter_by(id=id).one()
        try:
            db_session.delete(petition_document_ref_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def bulk_delete(document_id):
        try:
            db_session.query(
                PetitionDocumentReferenceModel).filter(
                PetitionDocumentReferenceModel.document_id == document_id
            ).delete(synchronize_session=False)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise


def _helper_create(data):
    new_petition_document_reference = PetitionDocumentReferenceModel(**data)
    try:
        db_session.add(new_petition_document_reference)
        db_session.commit()
        petition_document_ref_query = db_session.query(
            PetitionDocumentReferenceModel).get(
            new_petition_document_reference.id)
        response = schema_resource.dump(petition_document_ref_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _build_query(params):
    q = db_session.query(
        PetitionDocumentReferenceModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('document_id'):
        q = q.filter_by(document_id=params.get('document_id'))
    if params.get('type'):
        q = q.filter_by(type=params.get('type'))
    if params.get('value'):
        q = q.filter_by(value=params.get('value'))
    return q


def _is_petition_doc(document):
    match = re.search(r'petition(?:\s|$)', document.title, re.IGNORECASE)
    if document.type == 'petition' and match:
        return True
    return False



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class FileModel(Base):
    __tablename__ = 'files'

    id = Column(Integer, primary_key=True)
    file_format = Column(String(128), nullable=False)
    s3_bucket_name = Column(String(128), nullable=False)
    s3_key_name = Column(String(128), nullable=False)
    updated_at = Column(DateTime, nullable=False)

    __table_args__ = (
        UniqueConstraint(
            's3_bucket_name',
            's3_key_name',
        ),
    )



from marshmallow import ValidationError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from sqlalchemy.exc import IntegrityError
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.silver_surfer_models.resources.Trial import (
    Trial,
    TrialNotFoundException,
)


class TrialResourceTestCase(TestCase):
    def setUp(self):
        super(TrialResourceTestCase, self).setUp()
        self.inst = Trial()

        self.trial1 = self.inst.create({
            'ptab_trial_num': 'CTA-101',
            'trial_filing_date': '2017-05-06',
            'patent_num': '9999',
            'is_cleaned': False,
            'is_processed': False,
        })

    def test_create_validation_error_no_ptab(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {},
        )

    def test_create_validation_error_no_trial_filing_date(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'ptab_trial_num': 'CTA-102',
            },
        )

    def test_create_validation_error_no_patent_num(self):
        self.assertRaises(
            ValidationError,
            self.inst.create,
            {
                'ptab_trial_num': 'CTA-102',
                'trial_filing_date': '2017-05-06',
            },
        )

    def test_create_duplicate_ptab_trial_num(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'ptab_trial_num': 'CTA-101',
                'trial_filing_date': '2017-05-06',
                'patent_num': '1234',
                'is_cleaned': False,
                'is_processed': False,
            },
        )

    def test_create(self):
        data = {
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': '8888',
            'is_cleaned': False,
            'is_processed': False,
        }
        response = self.inst.create(data)
        second_equals_first(
            {
                'ptab_trial_num': data['ptab_trial_num'],
                'trial_filing_date': '2017-05-06T00:00:00+00:00',
                'patent_num': data['patent_num']
            },
            response,
        )

    def test_read_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.read,
            {'ptab_trial_num': ''},
        )

    def test_read_all(self):
        # setup
        self.inst.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': '1111',
            'is_cleaned': False,
            'is_processed': False,
        })
        response = self.inst.read({})
        self.assertEqual(len(response), 2)

    @parameterized.expand([
        ('ptab_trial_num', 'CTA-101', 1),
        ('trial_filing_date', '2017-05-08', 1),
        ('patent_num', '1111', 1),
    ])
    def test_read_with_params(self, query_field, query_value, expected_length):
        # setup
        self.inst.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-08',
            'patent_num': '1111',
            'is_cleaned': False,
            'is_processed': False,
        })
        response = self.inst.read({query_field: query_value})
        self.assertEqual(len(response), expected_length)

    def test_read_w_params_ids(self):
        resp = self.inst.read({'ids': []})
        self.assertEqual(0, len(resp))

        resp = self.inst.read({'ids': [self.trial1['id']]})
        self.assertEqual(1, len(resp))
        second_equals_first(
            self.trial1,
            resp[0]
        )

    @parameterized.expand([
        (['9999'], 1),
        (['1111'], 2),
        (['123'], 0),
    ])
    def test_read_with_param_patent_nums(
        self,
        patent_numbers,
        expected_length,
    ):
        # setup
        self.inst.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-08',
            'patent_num': '1111',
            'is_cleaned': False,
            'is_processed': False,
        })
        self.inst.create({
            'ptab_trial_num': 'CTA-103',
            'trial_filing_date': '2017-05-08',
            'patent_num': '1111',
            'is_cleaned': False,
            'is_processed': False,
        })
        response = self.inst.read({'patent_nums': patent_numbers})
        self.assertEqual(len(response), expected_length)

    def test_update_not_found(self):
        invalid_id = 99999
        args = []
        kwargs = {'id': invalid_id, 'params': {}}
        self.assertRaises(
            TrialNotFoundException,
            self.inst.update,
            *args,
            **kwargs
        )

    def test_update_validation_error_blank_ptab_trial_num(self):
        args = []
        kwargs = {
            'id': self.trial1['id'],
            'params': {
                'ptab_trial_num': '',
            },
        }
        self.assertRaises(
            ValidationError,
            self.inst.update,
            *args,
            **kwargs
        )

    def test_update(self):
        new_data = {'patent_num': '1000'}
        response = self.inst.update(
            id=self.trial1['id'],
            params=new_data,
        )
        second_equals_first(
            new_data,
            response,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            TrialNotFoundException,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        data = self.inst.read({'id': self.trial1['id']})
        self.assertEqual(len(data), 1)
        response = self.inst.delete(id=self.trial1['id'])
        self.assertEqual(response, 'Successfully deleted')
        data = self.inst.read({'id': self.trial1['id']})
        self.assertEqual(len(data), 0)

    def test_upsert_validation_error_missing_ptab_trial_num(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {},
        )

    def test_upsert_create_wo_patent_num(self):
        data = {
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
        }
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            data,
        )

    def test_upsert(self):
        data = {
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': '2222',
            'is_cleaned': False,
            'is_processed': False,
        }
        response = self.inst.upsert(params=data)
        second_equals_first(
            {
                'ptab_trial_num': data['ptab_trial_num'],
                'trial_filing_date': '2017-05-06T00:00:00+00:00',
                'patent_num': data['patent_num'],
            },
            response,
        )

        new_data = {
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-08',
            'patent_num': '3333',
            'is_cleaned': False,
            'is_processed': False,
        }
        response = self.inst.upsert(params=new_data)
        second_equals_first(
            {
                'ptab_trial_num': new_data['ptab_trial_num'],
                'trial_filing_date': '2017-05-08T00:00:00+00:00',
                'patent_num': new_data['patent_num'],
            },
            response,
        )

    def test_one_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.one,
            {'ptab_trial_num': ''},
        )

    def test_one_raises_multiple_results_found(self):
        # Adding trial with same `patent_num`
        self.inst.create({
            'ptab_trial_num': 'CTA-102',
            'trial_filing_date': '2017-05-06',
            'patent_num': self.trial1['patent_num'],
            'is_cleaned': False,
            'is_processed': False,
        })
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'patent_num': self.trial1['patent_num']},
        )

    def test_one_raises_no_result_found(self):
        invalid_patent_num = '1234'
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'patent_num': invalid_patent_num},
        )



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from .models import ProcessorResultModel
from .schemas import (
    ProcessorResultResourceSchema,
    ProcessorResultQueryParamsSchema,
)


schema_resource = ProcessorResultResourceSchema()
schema_params = ProcessorResultQueryParamsSchema()


class DBException(SQLAlchemyError):
    pass


class ProcessorResultNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class ProcessorResult:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                document_id: int: required
                processor_name: string: required
                result: string: required
                additional_data: dict: required
        :return:
            newly created object
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                document_id
                processor_name
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_query = _build_query(params=data)
        response = schema_resource.dump(patent_query, many=True).data
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            ProcessorResultNotFoundException
            DBException
        """

        processor_result_query = db_session.query(
            ProcessorResultModel).filter_by(
            id=id
        ).first()
        if not processor_result_query:
            raise ProcessorResultNotFoundException(
                'Processor result does not exist!',
            )
        try:
            db_session.delete(processor_result_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')


def _helper_create(data):
    new_processor_result = ProcessorResultModel(
        document_id=data['document_id'],
        processor_name=data['processor_name'],
        result=data['result'],
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_processor_result)
        db_session.commit()
        patent_query = db_session.query(
            ProcessorResultModel).get(new_processor_result.id)
        response = schema_resource.dump(patent_query).data
        db_session.close()
        return response
    except SQLAlchemyError as e:
        db_session.rollback()
        db_session.close()
        print(e)
        raise DBException('DB error')


def _build_query(params):
    q = db_session.query(
        ProcessorResultModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('document_id'):
        q = q.filter_by(document_id=params.get('document_id'))
    if params.get('processor_name'):
        q = q.filter_by(processor_name=params.get('processor_name'))
    return q



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.sql.operators import in_op
from datetime import datetime
from marshmallow import ValidationError

from ...database import db_session
from ...resources.Trial.models import TrialModel
from .schemas import (
    TrialResourceSchema,
    TrialQueryParamsSchema,
    TrialPatchSchema,
)
from ...utils.utils import update_model


schema_resource = TrialResourceSchema()
schema_params = TrialQueryParamsSchema()
schema_patch = TrialPatchSchema()


class DBException(SQLAlchemyError):
    pass


class TrialNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class Trial:
    @staticmethod
    def create(params):
        """
        :param
            TrialResourceSchema
        :return:
            TrialResourceSchema
        :exception:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            TrialQueryParamsSchema
        :return:
            List<TrialResourceSchema>
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        trial_query = _build_query(params=data)
        response = schema_resource.dump(trial_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: TrialQueryParamsSchema

        Returns: TrialResourceSchema
        Raises:
            ValidationError
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        trial_query = _build_query(params=data).one()
        response = schema_resource.dump(trial_query).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: TrialPatchSchema
        :return:
            TrialResourceSchema
        :exception:
            TrialNotFoundException
            ValidationError
            DBException
        """
        trial_query = db_session.query(TrialModel).filter_by(id=id).first()
        if not trial_query:
            raise TrialNotFoundException('Trial not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, trial_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            TrialNotFoundException
            DBException
        """

        trial_query = db_session.query(TrialModel).filter_by(id=id).first()
        if not trial_query:
            raise TrialNotFoundException('Trial not found!')
        try:
            db_session.delete(trial_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise DBException('DB error')

    @staticmethod
    def upsert(params):
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        trial_query = db_session.query(TrialModel).filter_by(
            ptab_trial_num=params.get('ptab_trial_num'),
        ).first()

        if trial_query is None:
            response = _helper_create(data)
        else:
            response = _helper_update(data, trial_query)
        return response


def _helper_create(data):
    new_trial = TrialModel(
        updated_at=datetime.utcnow(),
        **data,
    )
    try:
        db_session.add(new_trial)
        db_session.commit()
        trial_query = db_session.query(TrialModel).get(new_trial.id)
        response = schema_resource.dump(trial_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, trial_query):
    data['id'] = trial_query.id
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, trial_query)
        db_session.commit()
        response = schema_resource.dump(trial_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(TrialModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if 'ids' in params:
        q = q.filter(in_op(
            TrialModel.id,
            params['ids'],
        ))
    if params.get('ptab_trial_num'):
        q = q.filter_by(ptab_trial_num=params.get('ptab_trial_num'))
    if params.get('trial_filing_date'):
        q = q.filter_by(trial_filing_date=params.get('trial_filing_date'))
    if params.get('patent_num'):
        q = q.filter_by(patent_num=params.get('patent_num'))
    if 'patent_nums' in params:
        q = q.filter(in_op(
            TrialModel.patent_num,
            params.get('patent_nums'),
        ))
    if params.get('is_cleaned'):
        q = q.filter_by(is_cleaned=params.get('is_cleaned'))
    if params.get('is_processed'):
        q = q.filter_by(is_processed=params.get('is_processed'))
    return q
